'''.
           @@@@@@@@@@
       @@@@..........@@@@
    @@@         .        @@@
  @@.           .         . @@
 @  .     _     .         .   @
@........| |...................@    *********************************************
@      . | |   _____  .        @
@      . | |  |  __ \ .        @    La Data Web
@      . | |__| |  | |.   ***  @
@........|____| |  | |...*   *.@    Copyright © 2022 Ignacio Barrau
@   .       . | |__| |. *     *@
@   .       . |_____/ . *     *@    *********************************************
@   .       .         . *     *@
@   .       .         . *******@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
'''

import json
import requests
from simplepbi import utils
import pandas as pd

class Pipelines():
    """Simple library to use the Power BI api and obtain pipelines from it. The user must have administrator rights or assign permissions on the pipeline.
    """

    def __init__(self, token):
        """Create a simplePBI object to request pipelines API. The user must have administrator rights or assign permissions on the pipeline.
        *** THIS OBJECT IS IN PREVIEW IN SIMPLEPBI ***
        Args:
            token: String
                Bearer Token to use the Power Bi Rest API
        """
        self.token = token
            
    def get_pipeline(self, pipeline_id):
        """Returns the specified deployment pipeline.
        ### Parameters
        ----
        pipeline_id: str uuid
            The deployment pipeline ID
        ### Returns
        ----
        Dict:
            A dictionary containing all the pipelines in the organization.
        """
        try:
            url = "https://api.powerbi.com/v1.0/myorg/pipelines/{}".format(pipeline_id)
            res = requests.get(url, headers={'Content-Type': 'application/json', "Authorization": "Bearer {}".format(self.token)})
            res.raise_for_status()
            return res.json()
        except requests.exceptions.HTTPError as ex:
            print("HTTP Error: ", ex, "\nText: ", ex.response.text)
        except requests.exceptions.RequestException as e:
            print("Request exception: ", e)
            
    def get_pipelines(self):
        """Returns a list of deployment pipelines that the user has access to.
        ### Parameters
        ----
        None
        ### Returns
        ----
        Dict:
            A dictionary containing all the pipelines in the organization.
        """
        try:
            url = "https://api.powerbi.com/v1.0/myorg/pipelines"
            res = requests.get(url, headers={'Content-Type': 'application/json', "Authorization": "Bearer {}".format(self.token)})
            res.raise_for_status()
            return res.json()
        except requests.exceptions.HTTPError as ex:
            print("HTTP Error: ", ex, "\nText: ", ex.response.text)
        except requests.exceptions.RequestException as e:
            print("Request exception: ", e)
            
    def get_pipeline_operation(self, pipeline_id, operation_id):
        """Returns the details of the specified deploy operation performed on the specified deployment pipeline, including the deployment execution plan.
        ### Parameters
        ----
        pipeline_id: str uuid
            The deployment pipeline ID
        operation_id: str uuid
            The operation ID
        ### Returns
        ----
        Dict:
            A dictionary containing an operation of the pipeline in the organization.
        """
        try:
            url = "https://api.powerbi.com/v1.0/myorg/pipelines/{}/operations/{}".format(pipeline_id, operation_id)
            res = requests.get(url, headers={'Content-Type': 'application/json', "Authorization": "Bearer {}".format(self.token)})
            res.raise_for_status()
            return res.json()
        except requests.exceptions.HTTPError as ex:
            print("HTTP Error: ", ex, "\nText: ", ex.response.text)
        except requests.exceptions.RequestException as e:
            print("Request exception: ", e)
            
    def get_pipeline_operations(self, pipeline_id):
        """Returns a list of the up-to-20 most recent deploy operations performed on the specified deployment pipeline.
        ### Parameters
        ----
        pipeline_id: str uuid
            The deployment pipeline ID
        ### Returns
        ----
        Dict:
            A dictionary containing most recent operations of the deploy pipelines in the organization.
        """
        try:
            url = "https://api.powerbi.com/v1.0/myorg/pipelines/{}/operations".format(pipeline_id)
            res = requests.get(url, headers={'Content-Type': 'application/json', "Authorization": "Bearer {}".format(self.token)})
            res.raise_for_status()
            return res.json()
        except requests.exceptions.HTTPError as ex:
            print("HTTP Error: ", ex, "\nText: ", ex.response.text)
        except requests.exceptions.RequestException as e:
            print("Request exception: ", e)
        
    def get_pipeline_stage_artifacts(self, pipeline_id, stageOrder):
        """Returns the supported items from the workspace assigned to the specified stage of the specified deployment pipeline. To learn about items that aren't supported in deployment pipelines, see Unsupported items.
        ### Parameters
        ----
        pipeline_id: str uuid
            The deployment pipeline ID
        stageOrder: int32
            The deployment pipeline stage order. Development (0), Test (1), Production (2).
        ### Returns
        ----
        Dict:
            A dictionary containing items from the workspace assigned to the specified stage of the specified deployment pipeline.
        """
        try:
            url = "https://api.powerbi.com/v1.0/myorg/pipelines/{}/stages/{}/artifacts".format(pipeline_id, stageOrder)
            res = requests.get(url, headers={'Content-Type': 'application/json', "Authorization": "Bearer {}".format(self.token)})
            res.raise_for_status()
            return res.json()
        except requests.exceptions.HTTPError as ex:
            print("HTTP Error: ", ex, "\nText: ", ex.response.text)
        except requests.exceptions.RequestException as e:
            print("Request exception: ", e)
            
    def get_pipeline_stages(self, pipeline_id):
        """Returns the stages of the specified deployment pipeline.
        ### Parameters
        ----
        pipeline_id: str uuid
            The deployment pipeline ID
        ### Returns
        ----
        Dict:
            A dictionary containing stages of a deploy pipeline.
        """
        try:
            url = "https://api.powerbi.com/v1.0/myorg/pipelines/{}/stages".format(pipeline_id)
            res = requests.get(url, headers={'Content-Type': 'application/json', "Authorization": "Bearer {}".format(self.token)})
            res.raise_for_status()
            return res.json()
        except requests.exceptions.HTTPError as ex:
            print("HTTP Error: ", ex, "\nText: ", ex.response.text)
        except requests.exceptions.RequestException as e:
            print("Request exception: ", e)
        
    def get_pipeline_users(self, pipeline_id):
        """Returns a list of users that have access to the specified deployment pipeline.
        ### Parameters
        ----
        pipeline_id: str uuid
            The deployment pipeline ID
        ### Returns
        ----
        Dict:
            A dictionary containing a list of users with access to the deploy pipeline.
        """
        try:
            url = "https://api.powerbi.com/v1.0/myorg/pipelines/{}/users".format(pipeline_id)
            res = requests.get(url, headers={'Content-Type': 'application/json', "Authorization": "Bearer {}".format(self.token)})
            res.raise_for_status()
            return res.json()
        except requests.exceptions.HTTPError as ex:
            print("HTTP Error: ", ex, "\nText: ", ex.response.text)
        except requests.exceptions.RequestException as e:
            print("Request exception: ", e)            
            
    def assign_workspace_to_pipeline_preview(self, pipeline_id, stageOrder, workspace_id):
        """Assigns the specified workspace to the specified deployment pipeline stage.
        ### Parameters
        ----
        pipeline_id: str uuid
            The deployment pipeline ID
        stageOrder: int32
            The deployment pipeline stage order. Development (0), Test (1), Production (2).
        ### Request Body
        ----
        workspace_id: str uuid
            The workspace ID
        ### Returns
        ----
        Response object from requests library. 200 OK
        ### Limitations
        ----
        The specified deployment pipeline stage isn't already assigned.
        You must be an admin of the specified workspace.
        The specified workspace isn't assigned to any other deployment pipeline.
        This operation will fail if there's an active deployment operation.        
        """
        try: 
            url= "https://api.powerbi.com/v1.0/myorg/pipelines/{}/stages/{}/assignWorkspace".format(pipeline_id, stageOrder)
            body ={
                "workspaceId": workspace_id
            }                
            headers={'Content-Type': 'application/json', "Authorization": "Bearer {}".format(self.token)}            
            res = requests.post(url, data = json.dumps(body), headers = headers)
            res.raise_for_status()
            return res
        except requests.exceptions.HTTPError as ex:
            print("HTTP Error: ", ex, "\nText: ", ex.response.text)
        except requests.exceptions.RequestException as e:
            print("Request exception: ", e)
            
            
    def create_pipeline_preview(self, displayName, description=None):
        """Creates a new deployment pipeline.
        ### Parameters
        ----
        None
        ### Request Body
        ----
        displayName: str uuid
            The display name for the new deployment pipeline
        description: str
            The description for the new deployment pipeline
        ### Returns
        ----
        Response object from requests library. 201 Created OK     
        """
        try: 
            url= "https://api.powerbi.com/v1.0/myorg/pipelines"
            body ={
                "displayName": displayName
            }                
            if description != None:
                body["value"]["description"]=description
            headers={'Content-Type': 'application/json', "Authorization": "Bearer {}".format(self.token)}            
            res = requests.post(url, data = json.dumps(body), headers = headers)
            res.raise_for_status()
            return res
        except requests.exceptions.HTTPError as ex:
            print("HTTP Error: ", ex, "\nText: ", ex.response.text)
        except requests.exceptions.RequestException as e:
            print("Request exception: ", e)
            
    def delete_pipeline_preview(self, pipeline_id):
        """Deletes the specified deployment pipeline.
        ### Parameters
        ----
        pipeline_id: str uuid
            The deployment pipeline ID
        ### Returns
        ----
        Response object from requests library. 200 OK     
        
        """
        try:
            url = "https://api.powerbi.com/v1.0/myorg/pipelines/{}".format(pipeline_id)
            res = requests.delete(url, headers={'Content-Type': 'application/json', "Authorization": "Bearer {}".format(self.token)})
            res.raise_for_status()
            return res.json()
        except requests.exceptions.HTTPError as ex:
            print("HTTP Error: ", ex, "\nText: ", ex.response.text)
        except requests.exceptions.RequestException as e:
            print("Request exception: ", e)
            
    def delete_pipeline_user_preview(self, pipeline_id, identifier):
        """Removes user permissions from the specified deployment pipeline.
        ### Parameters
        ----
        pipeline_id: str uuid
            The deployment pipeline ID
        identifier: str
            To delete user pipeline permissions, provide the user principal name (UPN) of the user. To delete a service principal or a security group's pipeline permissions, provide the Object ID of the service principal or security group.
        ### Returns
        ----
        Response object from requests library. 200 OK     
        
        """
        try:
            url = "https://api.powerbi.com/v1.0/myorg/pipelines/{}/users/{}".format(pipeline_id, identifier)
            res = requests.delete(url, headers={'Content-Type': 'application/json', "Authorization": "Bearer {}".format(self.token)})
            res.raise_for_status()
            return res.json()
        except requests.exceptions.HTTPError as ex:
            print("HTTP Error: ", ex, "\nText: ", ex.response.text)
        except requests.exceptions.RequestException as e:
            print("Request exception: ", e)
            
    def unassign_workspace_preview(self, pipeline_id, stageOrder):
        """Unassigns the workspace from the specified stage in the specified deployment pipeline.
        ### Parameters
        ----
        pipeline_id: str uuid
            The deployment pipeline ID
        stageOrder: int32
            The deployment pipeline stage order. Development (0), Test (1), Production (2).
        ### Returns
        ----
        Response object from requests library. 200 OK
        
        """
        try: 
            url= "https://api.powerbi.com/v1.0/myorg/pipelines/{}/stages/{}/unassignWorkspace".format(pipeline_id, stageOrder)
                
            headers={'Content-Type': 'application/json', "Authorization": "Bearer {}".format(self.token)}
            
            res = requests.post(url, data = json.dumps(body), headers = headers)
            res.raise_for_status()
            return res
        except requests.exceptions.HTTPError as ex:
            print("HTTP Error: ", ex, "\nText: ", ex.response.text)
        except requests.exceptions.RequestException as e:
            print("Request exception: ", e)
            
    def deploy_all_preview(self, pipeline_id, sourceStageOrder, isBackwardDeployment=None, newWorkspace=None, options=None, updateAppSettings=None):
        """Deploys all supported items from the source stage of the specified deployment pipeline. To learn about items that aren't supported in deployment pipelines, see Unsupported items.
        ### Parameters
        ----
        pipeline_id: str uuid
            The deployment pipeline ID
        ### Request Body
        ----
        sourceStageOrder: integer
            The numeric identifier of the pipeline deployment stage that the content should be deployed from. Development (0), Test (1), Production (2).
        isBackwardDeployment: boolean
            Whether the deployment will be from a later stage in the deployment pipeline, to an earlier one. The default value is false.
        newWorkspace: PipelineNewWorkspaceRequest (Read more at Microsoft Docs to learn how to build the dict {})
            The configuration details for creating a new workspace. Required when deploying to a stage that has no assigned workspaces. The deployment will fail if the new workspace configuration details aren't provided when required.
        options: DeploymentOptions (Read more at Microsoft Docs to learn how to build the dict {})
            Options that control the behavior of the entire deployment
        updateAppSettings: PipelineUpdateAppSettings (Read more at Microsoft Docs to learn how to build the dict {})
            Update org app in the target workspace settings
        ### Returns
        ----
        Response object from requests library. 202 Accepted OK     
        ### Limitations
        ----
        Maximum 300 deployed items per request.
        """
        try: 
            url= "https://api.powerbi.com/v1.0/myorg/pipelines/{}/deployAll".format(pipeline_id)
            body ={
                "sourceStageOrder": sourceStageOrder
            }                
            if isBackwardDeployment != None:
                body["value"]["isBackwardDeployment"]=isBackwardDeployment
            if newWorkspace != None:
                body["value"]["newWorkspace"]=newWorkspace
            if options != None:
                body["value"]["options"]=options
            if updateAppSettings != None:
                body["value"]["updateAppSettings"]=updateAppSettings
            headers={'Content-Type': 'application/json', "Authorization": "Bearer {}".format(self.token)}            
            res = requests.post(url, data = json.dumps(body), headers = headers)
            res.raise_for_status()
            return res
        except requests.exceptions.HTTPError as ex:
            print("HTTP Error: ", ex, "\nText: ", ex.response.text)
        except requests.exceptions.RequestException as e:
            print("Request exception: ", e)
            
    def selective_deploy_preview(self, pipeline_id, sourceStageOrder, dashboards, dataflows, datamarts, datasets, isBackwardDeployment=None, newWorkspace=None, options=None, updateAppSettings=None):
        """Deploys the specified items from the source stage of the specified deployment pipeline.
        ### Parameters
        ----
        pipeline_id: str uuid
            The deployment pipeline ID
        ### Request Body
        ----
        sourceStageOrder: integer
            The numeric identifier of the pipeline deployment stage that the content should be deployed from. Development (0), Test (1), Production (2).
        dashboards: DeployArtifactRequest[]
            A list of dashboards to be deployed (Read more at Microsoft Docs to learn how to build the list [])
        dataflows: DeployArtifactRequest[]
            A list of dataflows to be deployed (Read more at Microsoft Docs to learn how to build the list [])
        datamarts: DeployArtifactRequest[]
            A list of datamarts to be deployed (Read more at Microsoft Docs to learn how to build the list [])
        datasets: DeployArtifactRequest[]
            A list of datasets to be deployed (Read more at Microsoft Docs to learn how to build the list [])
        isBackwardDeployment: boolean
            Whether the deployment will be from a later stage in the deployment pipeline, to an earlier one. The default value is false.
        newWorkspace: PipelineNewWorkspaceRequest (Read more at Microsoft Docs to learn how to build the dict {})
            The configuration details for creating a new workspace. Required when deploying to a stage that has no assigned workspaces. The deployment will fail if the new workspace configuration details aren't provided when required.
        options: DeploymentOptions (Read more at Microsoft Docs to learn how to build the dict {})
            Options that control the behavior of the entire deployment
        updateAppSettings: PipelineUpdateAppSettings (Read more at Microsoft Docs to learn how to build the dict {})
            Update org app in the target workspace settings
        ### Returns
        ----
        Response object from requests library. 202 Accepted OK     
        ### Limitations
        ----
        Maximum 300 deployed items per request.
        """
        try: 
            url= "https://api.powerbi.com/v1.0/myorg/pipelines/{}/deploy".format(pipeline_id)
            body ={
                "sourceStageOrder": sourceStageOrder
            }
            if dashboards != None:
                body["value"]["dashboards"]=dashboards
            if dataflows != None:
                body["value"]["dataflows"]=dataflows
            if datamarts != None:
                body["value"]["datamarts"]=datamarts
            if datasets != None:
                body["value"]["datasets"]=datasets
            if isBackwardDeployment != None:
                body["value"]["isBackwardDeployment"]=isBackwardDeployment
            if newWorkspace != None:
                body["value"]["newWorkspace"]=newWorkspace
            if options != None:
                body["value"]["options"]=options
            if updateAppSettings != None:
                body["value"]["updateAppSettings"]=updateAppSettings
            headers={'Content-Type': 'application/json', "Authorization": "Bearer {}".format(self.token)}            
            res = requests.post(url, data = json.dumps(body), headers = headers)
            res.raise_for_status()
            return res
        except requests.exceptions.HTTPError as ex:
            print("HTTP Error: ", ex, "\nText: ", ex.response.text)
        except requests.exceptions.RequestException as e:
            print("Request exception: ", e)
            
    def update_pipeline_preview(self, pipeline_id, displayName, description=None):
        """Updates the specified deployment pipeline.
        ### Parameters
        ----
        pipeline_id: str uuid
            The deployment pipeline ID
        ### Request Body
        ----
        displayName: str uuid
            The display name for the new deployment pipeline
        description: str
            The description for the new deployment pipeline
        ### Returns
        ----
        Response object from requests library. 200 OK     
        """
        try: 
            url= "https://api.powerbi.com/v1.0/myorg/pipelines/{}".format(pipeline_id)
            body ={
                "displayName": displayName
            }                
            if description != None:
                body["value"]["description"]=description
            headers={'Content-Type': 'application/json', "Authorization": "Bearer {}".format(self.token)}            
            res = requests.patch(url, data = json.dumps(body), headers = headers)
            res.raise_for_status()
            return res
        except requests.exceptions.HTTPError as ex:
            print("HTTP Error: ", ex, "\nText: ", ex.response.text)
        except requests.exceptions.RequestException as e:
            print("Request exception: ", e)
            
    def update_pipeline_user_preview(self, pipeline_id, identifier, principalType, accessRight):
        """Updates the specified deployment pipeline.
        ### Parameters
        ----
        pipeline_id: str uuid
            The deployment pipeline ID
        ### Request Body
        ----
        identifier: str
            For principal type User, provide the UPN. Otherwise provide the object ID of the principal.
        principalType: str
            The principal type. E.g { "App", "Group", "None", "User" }
        accessRight: str
            Required. The access right a user has for the deployment pipeline. E.g "Admin"
        ### Returns
        ----
        Response object from requests library. 200 OK     
        """
        try: 
            url= "https://api.powerbi.com/v1.0/myorg/pipelines/{}/users".format(pipeline_id)
            body ={
                "identifier": identifier,
                "accessRight": accessRight,
                "principalType": principalType
            }
            headers={'Content-Type': 'application/json', "Authorization": "Bearer {}".format(self.token)}            
            res = requests.post(url, data = json.dumps(body), headers = headers)
            res.raise_for_status()
            return res
        except requests.exceptions.HTTPError as ex:
            print("HTTP Error: ", ex, "\nText: ", ex.response.text)
        except requests.exceptions.RequestException as e:
            print("Request exception: ", e)


'''.
           @@@@@@@@@@
       @@@@..........@@@@
    @@@         .        @@@
  @@.           .         . @@
 @  .     _     .         .   @
@........| |...................@    *********************************************
@      . | |   _____  .        @
@      . | |  |  __ \ .        @    La Data Web
@      . | |__| |  | |.   ***  @
@........|____| |  | |...*   *.@    Copyright © 2022 Ignacio Barrau
@   .       . | |__| |. *     *@
@   .       . |_____/ . *     *@    *********************************************
@   .       .         . *     *@
@   .       .         . *******@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
'''

import json
import requests

class Dataflows():
    """Simple library to use the Power BI api and obtain dataflows from it.
    """

    def __init__(self, token):
        """Create a simplePBI object to request admin API
        Args:
            token: String
                Bearer Token to use the Power Bi Rest API
        """
        self.token = token
               
    def get_dataflow_in_group(self, workspace_id, dataflow_id):
        """Returns a single dataflow from the specified workspace.
        ### Parameters
        ----
        workspace_id: str uuid
            The Power Bi workspace id. You can take it from PBI Service URL
        dataflow_id: str uuid
            The Power Bi dataflow id. You can take it from PBI Service URL
        ### Returns
        ----
        Dict:
            A dictionary containing a dataflow in the workspace.
        """
        try:
            url = "https://api.powerbi.com/v1.0/myorg/groups/{}/dataflows/{}".format(workspace_id, dataflow_id)
            res = requests.get(url, headers={'Content-Type': 'application/json', "Authorization": "Bearer {}".format(self.token)})
            res.raise_for_status()
            return res.json()
        except requests.exceptions.HTTPError as ex:
            print("HTTP Error: ", ex, "\nText: ", ex.response.text)
        except requests.exceptions.RequestException as e:
            print("Request exception: ", e)
                    
    def get_dataflows_in_group(self, workspace_id):
        """Returns a list of all dataflows from the specified workspace.
        ### Parameters
        ----
        workspace_id: str uuid
            The Power Bi workspace id. You can take it from PBI Service URL
        ### Returns
        ----
        Dict:
            A dictionary containing all the dataflows in the workspace.
        """
        try:
            url = "https://api.powerbi.com/v1.0/myorg/groups/{}/dataflows".format(workspace_id)
            res = requests.get(url, headers={'Content-Type': 'application/json', "Authorization": "Bearer {}".format(self.token)})
            res.raise_for_status()
            return res.json()
        except requests.exceptions.HTTPError as ex:
            print("HTTP Error: ", ex, "\nText: ", ex.response.text)
        except requests.exceptions.RequestException as e:
            print("Request exception: ", e)

            
    def get_datasources_in_group(self, workspace_id, dataflow_id):
        """Returns a list of datasources for the specified dataflow.
        ### Parameters
        ----
        workspace_id: str uuid
            The Power Bi workspace id. You can take it from PBI Service URL
        dataflow_id: str uuid
            The Power Bi dataflow id. You can take it from PBI Service URL
        ### Returns
        ----
        Dict:
            A dictionary containing all the datasources in the dataflow from the workspace.
        """
        try:
            url = "https://api.powerbi.com/v1.0/myorg/groups/{}/dataflows/{}/datasources".format(workspace_id, dataflow_id)
            res = requests.get(url, headers={'Content-Type': 'application/json', "Authorization": "Bearer {}".format(self.token)})
            res.raise_for_status()
            return res.json()
        except requests.exceptions.HTTPError as ex:
            print("HTTP Error: ", ex, "\nText: ", ex.response.text)
        except requests.exceptions.RequestException as e:
            print("Request exception: ", e)

            
    def get_upstream_dataflow_in_group(self, workspace_id, dataflow_id):
        """Returns a list of upstream dataflows for dataflows from the specified workspace.
        ### Parameters
        ----
        workspace_id: str uuid
            The Power Bi workspace id. You can take it from PBI Service URL
        ### Returns
        ----
        Dict:
            A dictionary containing all upstream dataflows in the dataflow from a workspace
        """
        try:
            url = "https://api.powerbi.com/v1.0/myorg/groups/{}/dataflows/{}/upstreamDataflows".format(workspace_id, dataflow_id)
            res = requests.get(url, headers={'Content-Type': 'application/json', "Authorization": "Bearer {}".format(self.token)})
            res.raise_for_status()
            return res.json()
        except requests.exceptions.HTTPError as ex:
            print("HTTP Error: ", ex, "\nText: ", ex.response.text)
        except requests.exceptions.RequestException as e:
            print("Request exception: ", e)

                      
    def refresh_dataflow_in_group(self, workspace_id, dataflow_id, notifyOption, processType=None):
        """Triggers a refresh for the specified dataflow. The only supported mail notification options are either in case of failure, or none. MailOnCompletion is not supported.

        ### Parameters
        ----
        workspace_id: str uuid
            The Power Bi workspace id. You can take it from PBI Service URL        
        dataflow_id: str uuid
            The Power Bi dataflow id. You can take it from PBI Service URL
        processType: str uuid
            Type of refresh process to use.
        ### Request Body
        ----
        notifyOption: NotifyOption str
            Mail notification options (success and/or failure, or none). Options: { MailOnCompletion, MailOnFailure, NoNotification }
        ### Returns
        ----
        Response object from requests library. 202 OK
        
        """
        try: 
            url= "https://api.powerbi.com/v1.0/myorg/groups/{}/dataflows/{}/refreshes".format(workspace_id, dataflow_id)
            if processType != None:
                url = url + "?processType={" + processType + "}"
            body = {
                "notifyOption": notifyOption 
            }               
            headers={'Content-Type': 'application/json', "Authorization": "Bearer {}".format(self.token)}
            res = requests.post(url, data = json.dumps(body), headers = headers)
            res.raise_for_status()
            return res
        except requests.exceptions.HTTPError as ex:
            print("HTTP Error: ", ex, "\nText: ", ex.response.text)
        except requests.exceptions.RequestException as e:
            print("Request exception: ", e)

                        
    def delete_dataflow_in_group(self, workspace_id, dataflow_id):
        """Deletes the specified dataflow from the specified workspace.
        ### Parameters
        ----
        dataflow_id: str uuid
            The Power Bi dataflow id. You can take it from PBI Service URL
        ### Returns
        ----
        Response object from requests library. 200 OK
        
        """
        try: 
            url= "https://api.powerbi.com/v1.0/myorg/groups/{}/dataflows/{}".format(workspace_id, dataflow_id)   
            headers={'Content-Type': 'application/json', "Authorization": "Bearer {}".format(self.token)}
            res = requests.delete(url, headers=headers)
            res.raise_for_status()
            return res
        except requests.exceptions.HTTPError as ex:
            print("HTTP Error: ", ex, "\nText: ", ex.response.text)
        except requests.exceptions.RequestException as e:
            print("Request exception: ", e)

                        
    def update_refresh_schedule_in_group_preview(self, workspace_id, dataflow_id, NotifyOption=None, days=None, enabled=None, localTimeZoneId=None, times=None):
        """Creates or updates the specified dataflow refresh schedule configuration.    
        ### Parameters
        ----
        workspace_id: str uuid
            The Power Bi workspace id. You can take it from PBI Service URL
        dataflow_id: str uuid
            The Power Bi dataflow id. You can take it from PBI Service URL
        ### Request Body
        ----
        NotifyOption: ShceduleNotifyOption str
            Notification option at scheduled refresh termination. Example MailOnFailure or NoNotification.
        days: str []
            Days to execute the refresh. Example: ["Sunday", "Tuesday"]
        enabled: bool
            is the refresh enabled
        localTimeZoneId: str
            The ID of the timezone to use. See TimeZone Info. Example "UTC"
        times: str []
            Times to execute the refresh within each day. Example: ["07:00", "16:00"]
        ### Returns
        ----
        Response object from requests library. 200 OK
        ### Limitations
        ----
        The limit on the number of time slots per day depends on whether a Premium or Shared capacity is used.
        """
        try: 
            url= "https://api.powerbi.com/v1.0/myorg/groups/{}/dataflows/{}/refreshSchedule".format(workspace_id, dataflow_id)
            body = {
                "value": {}
            }
            
            if NotifyOption != None:
                body["value"]["NotifyOption"]=NotifyOption
            if days != None:
                body["value"]["days"] = days
            if enabled != None:
                body["value"]["enabled"] = enabled
            if localTimeZoneId != None:
                body["value"]["localTimeZoneId"] = localTimeZoneId
            if times != None:
                body["value"]["times"]=times
                
            headers={'Content-Type': 'application/json', "Authorization": "Bearer {}".format(self.token)}
            
            res = requests.patch(url, json.dumps(body), headers = headers)
            res.raise_for_status()
            return res
        except requests.exceptions.HTTPError as ex:
            print("HTTP Error: ", ex, "\nText: ", ex.response.text)
        except requests.exceptions.RequestException as e:
            print("Request exception: ", e)

            
    def update_dataflow_preview(self, workspace_id, dataflow_id, allowNativeQueries=None, computeEngineBehavior=None, description=None, name=None):
        """Update dataflow properties, capabilities and settings.
        ### Parameters
        ----
        workspace_id: str uuid
            The Power Bi workspace id. You can take it from PBI Service URL
        dataflow_id: str uuid
            The Power Bi dataflow id. You can take it from PBI Service URL
        ### Request Body
        ----
        allowNativeQueries: bool
            Allow native queries
        computeEngineBehavior: str 
            Compute Engine Behavior. Examples: "computeOptimized", "computeOn" or "computeDisabled"
        description: str
            New description for the dataflow
        name: str
            New name for the dataflow
        ### Returns
        ----
        Response object from requests library. 200 OK
        ### Limitations
        ----
        The limit on the number of time slots per day depends on whether a Premium or Shared capacity is used.
        """
        try: 
            url= "https://api.powerbi.com/v1.0/myorg/groups/{}/dataflows/{}/refreshSchedule".format(workspace_id, dataflow_id)
            body = {                
            }
            
            if allowNativeQueries != None:
                body["allowNativeQueries"] = allowNativeQueries
            if computeEngineBehavior != None:
                body["computeEngineBehavior"] = computeEngineBehavior
            if description != None:
                body["description"] = description
            if name != None:
                body["name"] = name
                
            headers={'Content-Type': 'application/json', "Authorization": "Bearer {}".format(self.token)}
            
            res = requests.patch(url, json.dumps(body), headers = headers)
            res.raise_for_status()
            return res
        except requests.exceptions.HTTPError as ex:
            print("HTTP Error: ", ex, "\nText: ", ex.response.text)
        except requests.exceptions.RequestException as e:
            print("Request exception: ", e)

            
    def get_dataflow_transaction_in_group(self, workspace_id, dataflow_id):
        """Returns a list of transactions for the specified dataflow.
        ### Parameters
        ----
        workspace_id: str uuid
            The Power Bi workspace id. You can take it from PBI Service URL
        dataflow_id: str uuid
            The Power Bi dataflow id. You can take it from PBI Service URL
        ### Returns
        ----
        Dict:
            A dictionary containing all the transactions in the dataflow from the workspace.
        """
        try:
            url = "https://api.powerbi.com/v1.0/myorg/groups/{}/dataflows/{}/transactions".format(workspace_id, dataflow_id)
            res = requests.get(url, headers={'Content-Type': 'application/json', "Authorization": "Bearer {}".format(self.token)})
            res.raise_for_status()
            return res.json()
        except requests.exceptions.HTTPError as ex:
            print("HTTP Error: ", ex, "\nText: ", ex.response.text)
        except requests.exceptions.RequestException as e:
            print("Request exception: ", e)

            
    def cancel_dataflow_transaction_in_group_preview(self, workspace_id, dataflow_id, transaction_id):
        """Attempts to Cancel the specified transactions.
        ### Parameters
        ----
        workspace_id: str uuid
            The Power Bi workspace id. You can take it from PBI Service URL        
        dataflow_id: str uuid
            The Power Bi dataflow id. You can take it from PBI Service URL
        transaction_id: str uuid
            The transaction id. You can get it from get_dataflow_transaction_in_group
        ### Returns
        ----
        Response object from requests library. 202 OK
        
        """
        try: 
            url= "https://api.powerbi.com/v1.0/myorg/groups/{}/dataflows/{}/transactions/{}/cancel".format(workspace_id, dataflow_id, transaction_id)
            if url != None:
                url = url + "?processType={" + processType + "}"           
            headers={'Content-Type': 'application/json', "Authorization": "Bearer {}".format(self.token)}
            res = requests.post(url, headers = headers)
            res.raise_for_status()
            return res
        except requests.exceptions.HTTPError as ex:
            print("HTTP Error: ", ex, "\nText: ", ex.response.text)
        except requests.exceptions.RequestException as e:
            print("Request exception: ", e)



'''.
           @@@@@@@@@@
       @@@@..........@@@@
    @@@         .        @@@
  @@.           .         . @@
 @  .     _     .         .   @
@........| |...................@    *********************************************
@      . | |   _____  .        @
@      . | |  |  __ \ .        @    La Data Web
@      . | |__| |  | |.   ***  @
@........|____| |  | |...*   *.@    Copyright © 2022 Ignacio Barrau
@   .       . | |__| |. *     *@
@   .       . |_____/ . *     *@    *********************************************
@   .       .         . *     *@
@   .       .         . *******@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
'''

from setuptools import find_packages, setup

classifiers = [
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
]

setup(
    name='simplepbi',
    packages=find_packages(),
    version='1.0.6',
    project_urls={
        'Documentation': 'https://docs.microsoft.com/en-us/rest/api/power-bi/',
        'Say Thanks!': 'https://www.ladataweb.com.ar/contacto.html',
        'Source': 'https://github.com/ladataweb/SimplePBI',
        'Tracker': 'https://github.com/ladataweb/SimplePBI/issues'
    },
    download_url='',
    url='',
    description='Simplify usage of Power Bi Rest API',
    long_description=open('README.md').read() + '\n\n' + open('CHANGELOG.txt').read(),
    long_description_content_type="text/markdown",
    author='Ignacio Barrau <igna_barrau@hotmail.com>, Martin Zurita <martinzurita1@gmail.com>',
    license='MIT',
    classifiers=classifiers,
    install_requires=[
        'requests', 
        'pandas',
        'requests-toolbelt'
    ],
    keywords=['Power BI Rest API', 'Power BI', 'Power Bi API', 'PBI', 'LaDataWeb', 'Azure', 'Data', 'Python', 'Fabric', 'Microsoft Fabric', 'Fabric Rest API', 'Fabric API']
)



'''.
           @@@@@@@@@@
       @@@@..........@@@@
    @@@         .        @@@
  @@.           .         . @@
 @  .     _     .         .   @
@........| |...................@    *********************************************
@      . | |   _____  .        @
@      . | |  |  __ \ .        @    La Data Web
@      . | |__| |  | |.   ***  @
@........|____| |  | |...*   *.@    Copyright © 2022 Ignacio Barrau
@   .       . | |__| |. *     *@
@   .       . |_____/ . *     *@    *********************************************
@   .       .         . *     *@
@   .       .         . *******@
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
'''

import json
import requests

class Groups():
    """Simple library to use the Power BI api and obtain groups from it.
    """

    def __init__(self, token):
        """Create a simplePBI object to request admin API
        Args:
            token: String
                Bearer Token to use the Power Bi Rest API
        """
        self.token = token
        
    def add_user_group(self, workspace_id, groupUserAccessRight, emailAddress, displayName=None, graphId=None, identifier=None, principalType=None):
        """Grants the specified user the specified permissions to the specified workspace.
        When user permissions to a workspace have been recently updated, the new permissions might not be immediately available through API calls. To refresh user permissions, use the Refresh User Permissions API call.
        ### Parameters
        ----
        workspace_id: str uuid
            The Power Bi workspace id. You can take it from PBI Service URL
        ### Request Body
        ----
        groupUserAccessRight: GroupUserAccessRight
            Access rights user has for the workspace (Permission level: Admin, Contributor, Member, Viewer or None). This is mandatory
        emailAddress: str
            Email address of the user. This is mandatory.
        displayName: str
            Display name of the principal
        graphId: str
            Identifier of the principal in Microsoft Graph. Only available for admin APIs
        identifier: str
            Object ID of the principal
        principalType: principalType
            The principal type (App, Group, User or None)
        ### Returns
        ----
        Response object from requests library. 200 OK
        
        """
        try: 
            url= "https://api.powerbi.com/v1.0/myorg/groups/{}/users".format(workspace_id)
            body = {
                "groupUserAccessRight": groupUserAccessRight, 
                "emailAddress": emailAddress 
            }
            if displayName != None:
                body["diplayName"]=displayName
            if graphId != None:
                body["graphId"] = graphId
            if identifier != None:
                body["identifier"] = identifier
            if principalType != None:
                body["principalType"] = principalType
                
            headers={'Content-Type': 'application/json', "Authorization": "Bearer {}".format(self.token)}
            
            res = requests.post(url, data = json.dumps(body), headers = headers)
            res.raise_for_status()
            return res
        except requests.exceptions.HTTPError as ex:
            print("HTTP Error: ", ex, "\nText: ", ex.response.text)
        except requests.exceptions.RequestException as e:
            print("Request exception: ", e)

    def get_groups(self):
        """Returns a list of workspaces the user has access to.
        When user permissions to a workspace have been recently updated, the new permissions might not be immediately available through API calls. To refresh user permissions, use the Refresh User Permissions API call.
        ### Parameters
        ----
        None
        ### Returns
        ----
        Dict:
            A dictionary containing the groups of the user.
        """
        try:
            url = "https://api.powerbi.com/v1.0/myorg/groups"
            res = requests.get(url, headers={'Content-Type': 'application/json', "Authorization": "Bearer {}".format(self.token)})
            res.raise_for_status()
            return res.json()
        except requests.exceptions.HTTPError as ex:
            print("HTTP Error: ", ex, "\nText: ", ex.response.text)
        except requests.exceptions.RequestException as e:
            print("Request exception: ", e)
            
    def get_groups_users(self, workspace_id):
        """Returns a list of users that have access to the specified workspace.
        When user permissions to a workspace have been recently updated, the new permissions might not be immediately available through API calls. As a result, this API call might return an HTTP 401 error when a user has permissions to a workspace. To refresh user permissions, use the Refresh User Permissions API call.
        ### Parameters
        ----
        workspace_id: str uuid
            The Power Bi Workspace id. You can take it from PBI Service URL.
        ### Returns
        ----
        Dict:
            A dictionary containing all the users in the workspace.
        """
        try:
            url = "https://api.powerbi.com/v1.0/myorg/groups/{}/users".format(workspace_id)
            res = requests.get(url, headers={'Content-Type': 'application/json', "Authorization": "Bearer {}".format(self.token)})
            res.raise_for_status()
            return res.json()
        except requests.exceptions.HTTPError as ex:
            print("HTTP Error: ", ex, "\nText: ", ex.response.text)
        except requests.exceptions.RequestException as e:
            print("Request exception: ", e)
            
    def delete_group_preview(self, workspace_id):
        """Deletes the specified workspace.
        ### Parameters
        ----
        workspace_id: str uuid
            The Power Bi group id. You can take it from PBI Service URL
        ### Returns
        ----
        Response object from requests library. 200 OK
        
        """
        try: 
            url= "https://api.powerbi.com/v1.0/myorg/groups/{}".format(workspace_id)   
            headers={'Content-Type': 'application/json', "Authorization": "Bearer {}".format(self.token)}
            res = requests.delete(url, headers=headers)
            res.raise_for_status()
            return res
        except requests.exceptions.HTTPError as ex:
            print("HTTP Error: ", ex, "\nText: ", ex.response.text)
        except requests.exceptions.RequestException as e:
            print("Request exception: ", e)
            
    def delete_user_in_group(self, workspace_id, user):
        """Deletes the specified user permissions from the specified workspace.
        ### Parameters
        ----
        workspace_id: str uuid
            The Power Bi group id. You can take it from PBI Service URL
        user: str
            The email address of the user or object ID of the service principal to delete.
        ### Returns
        ----
        Response object from requests library. 200 OK
        
        """
        try: 
            url= "https://api.powerbi.com/v1.0/myorg/groups/{}/users/{}".format(workspace_id, user)   
            headers={'Content-Type': 'application/json', "Authorization": "Bearer {}".format(self.token)}
            res = requests.delete(url, headers=headers)
            res.raise_for_status()
            return res
        except requests.exceptions.HTTPError as ex:
            print("HTTP Error: ", ex, "\nText: ", ex.response.text)
        except requests.exceptions.RequestException as e:
            print("Request exception: ", e)

    def create_group_preview(self, name, workspaceV2=None):
        """Creates a new workspace.
        ### Parameters
        ----
        workspaceV2: bool
            Preview feature: Create a workspace V2. The only supported value is true.
        ### Request Body
        ----
        name: str
            The name of the newly created group.
        ### Returns
        ----
        Response object from requests library. 200 OK
        
        """
        try: 
            url= "https://api.powerbi.com/v1.0/myorg/groups"
            if workspaceV2 != None:
                url = url + "?workspaceV2={}".format(workspaceV2)
            body = {
                "name": name
            }                
            headers={'Content-Type': 'application/json', "Authorization": "Bearer {}".format(self.token)}
            
            res = requests.post(url, data = json.dumps(body), headers = headers)
            res.raise_for_status()
            return res
        except requests.exceptions.HTTPError as ex:
            print("HTTP Error: ", ex, "\nText: ", ex.response.text)
        except requests.exceptions.RequestException as e:
            print("Request exception: ", e)
            
    def get_dataflow_storage_account(self):
        """Returns a list of dataflow storage accounts that the user has access to.        
        ### Returns
        ----
        Dict:
            A dictionary containing all the dataflow storage accounts that the user has access to.
        """
        try:
            url = "https://api.powerbi.com/v1.0/myorg/dataflowStorageAccounts"
            res = requests.get(url, headers={'Content-Type': 'application/json', "Authorization": "Bearer {}".format(self.token)})
            res.raise_for_status()
            return res.json()
        except requests.exceptions.HTTPError as ex:
            print("HTTP Error: ", ex, "\nText: ", ex.response.text)
        except requests.exceptions.RequestException as e:
            print("Request exception: ", e)            

    def assign_to_dataflow_storage(self, workspace_id, dataflow_storage_id):
        """Assigns the specified workspace to the specified dataflow storage account.
        To perform this operation, the user must be an admin on the specified workspace and the Power BI dataflow storage account must be enabled.
        To unassign the specified workspace from a Power BI dataflow storage account, provide an empty GUID (00000000-0000-0000-0000-000000000000) as the dataflowStorageId.
        ### Parameters
        ----
        workspace_id: str uuid
            The Power Bi workspace id. You can take it from PBI Service URL        
        ### Request Body
        ----
        dataflow_storage_id: str uuid
            Mail notification options (success and/or failure, or none). Options: { MailOnCompletion, MailOnFailure, NoNotification }
        ### Returns
        ----
        Response object from requests library. 200 OK
        
        """
        try: 
            url= "https://api.powerbi.com/v1.0/myorg/groups/{}/AssignToDataflowStorage".format(workspace_id)
            body = {
                "dataflowStorageId": dataflow_storage_id 
            }               
            headers={'Content-Type': 'application/json', "Authorization": "Bearer {}".format(self.token)}
            res = requests.post(url, data = json.dumps(body), headers = headers)
            res.raise_for_status()
            return res
        except requests.exceptions.HTTPError as ex:
            print("HTTP Error: ", ex, "\nText: ", ex.response.text)
        except requests.exceptions.RequestException as e:
            print("Request exception: ", e)
            
            
            
''' REQUEST not working
    def update_user_group_preview(self, workspace_id, groupUserAccessRight, emailAddress, displayName=None, graphId=None, identifier=None, principalType=None):
        """Grants the specified user the specified permissions to the specified workspace.
        ***This request is IN PREVIEW, it can contain fails yet.***
        When user permissions to a workspace have been recently updated, the new permissions might not be immediately available through API calls. To refresh user permissions, use the Refresh User Permissions API call.
        ### Parameters
        ----
        workspace_id: str uuid
            The Power Bi workspace id. You can take it from PBI Service URL
        ### Request Body
        ----
        groupUserAccessRight: GroupUserAccessRight
            Access rights user has for the workspace (Permission level: Admin, Contributor, Member, Viewer or None). This is mandatory
        emailAddress: str
            Email address of the user. This is mandatory.
        displayName: str
            Display name of the principal
        graphId: str
            Identifier of the principal in Microsoft Graph. Only available for admin APIs
        identifier: str
            Object ID of the principal
        principalType: principalType
            The principal type (App, Group, User or None). This is mandatory
        ### Returns
        ----
        Response object from requests library. 200 OK
        
        """
        try: 
            url= "https://api.powerbi.com/v1.0/myorg/groups/{}/users".format(workspace_id)
            body = {
                "groupUserAccessRight": groupUserAccessRight, 
                "emailAddress": emailAddress,
                "principalType": principalType
            }
            if displayName != None:
                body["diplayName"]=displayName
            if graphId != None:
                body["graphId"] = graphId
            if identifier != None:
                body["identifier"] = identifier
                
            headers={'Content-Type': 'application/json', "Authorization": "Bearer {}".format(self.token)}
            
            res = requests.put(url, data = json.dumps(body), headers = headers)
            res.raise_for_status()
            return res
        except requests.exceptions.HTTPError as ex:
            print("HTTP Error: ", ex, "\nText: ", ex.response.text)
        except requests.exceptions.RequestException as e:
            print("Request exception: ", e)
'''    
